!-------------------------------------------------------------------------------

!                      Code_Saturne version 5.0.8
!                      --------------------------
! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2018 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------

!===============================================================================
! Function:
! ---------

!> \file cs_user_boundary_conditions_ale.f90
!>
!> \brief User subroutine dedicated the use of ALE (Arbitrary Lagrangian
!> Eulerian) Method:
!>  - Fills boundary conditions (ialtyb, icodcl, rcodcl) for mesh velocity.
!>  - This subroutine also enables one to fix displacement on nodes.
!>
!> See \subpage cs_user_boundary_conditions_ale for examples.
!>
!> \section intro_usa Introduction
!>
!> Here one defines boundary conditions on a per-face basis.
!>
!> Boundary faces may be identified using the \ref getfbr subroutine.
!> The syntax of this subroutine is described in
!> cs_user_boundary_conditions.f90 subroutine,
!> but a more thorough description can be found in the user guide.
!>
!> Boundary conditions setup for standard variables (pressure, velocity,
!> turbulence, scalars) is described precisely in
!> cs_user_boundary_conditions.f90 subroutine.
!>
!> Detailed explanation will be found in the theory guide.
!>
!> \section bc_types_usa Boundary condition types
!>
!> Boundary conditions may be assigned in two ways.
!>
!>
!> \subsection std_bcs_usa For "standard" boundary conditions
!>
!> One defines a code in the \c ialtyb array (of dimensions number of
!> boundary faces). The available codes are:
!>
!>  - \c ialtyb(ifac) = \c ibfixe: the face \c ifac is considered to be motionless.
!>           A zero Dirichlet boundary condition is automatically imposed on mesh
!>           velocity. Moreover the displacement of corresponding nodes will
!>           automatically be set to 0 (for further information please
!>           read the paragraph dedicated to the description of \c impale array
!>           in the usalcl.f90 subroutine), unless the USER has modified the
!>           condition of at least one mesh velocity component (modification of
!>           \c icodcl array, please read the following paragraph
!>           \ref non_std_bc_usa).
!>
!>  - \c ialtyb(ifac) = \c igliss: The mesh slides on corresponding face \c ifac.
!>           The normal component of mesh velocity is automatically set to 0.
!>           A homogeneous Neumann condition is automatically prescribed for the
!>           other components, as it's the case for 'Symmetry' fluid condition.
!>
!>  - \c ialtyb(ifac) = \c ivimpo: the mesh velocity is imposed on face \c ifac. Thus,
!>           the users needs to specify the mesh velocity values filling \c rcodcl
!>           arrays as follows:
!>            - \c rcodcl(ifac,iuma,1) = mesh velocity in 'x' direction
!>            - \c rcodcl(ifac,ivma,1) = mesh velocity in 'y' direction
!>            - \c rcodcl(ifac,iwma,1) = mesh velocity in 'z' direction
!>            .
!>           Components of \c rcodcl(.,i.ma,1) arrays that are not specified by user
!>           will automatically be set to 0, meaning that user only needs to specify
!>           non zero mesh velocity components.
!>
!>
!> \subsection non_std_bc_usa For "non-standard" conditions
!>
!> Other than (fixed boundary, sliding mesh boundary, fixed velocity), one
!> defines for each face and each component \c IVAR = IUMA, IVMA, IWMA:
!>  - a code
!>    - \c icodcl(ifac, ivar)
!>  - three real values:
!>    - \c rcodcl(ifac, ivar, 1)
!>    - \c rcodcl(ifac, ivar, 2)
!>    - \c rcodcl(ifac, ivar, 3)
!>
!> The value of \c icodcl is taken from the following:
!>  - 1: Dirichlet
!>  - 3: Neumann
!>  - 4: Symmetry
!>
!> The values of the 3 \c rcodcl components are:
!>  - \c rcodcl(ifac, ivar, 1):
!>     Dirichlet for the variable if \c icodcl(ifac, ivar) = 1
!>     The dimension of \c rcodcl(ifac, ivar, 1) is in m/s
!>  - \c rcodcl(ifac, ivar, 2):
!>    "exterior" exchange coefficient (between the prescribed value
!>                      and the value at the domain boundary)
!>                      rinfin = infinite by default
!>    \c  rcodcl(ifac,ivar,2) =  (VISCMA) / d
!>          (D has the dimension of a distance in m, VISCMA stands for
!>          the mesh viscosity)
!>    \remark the definition of \c rcodcl(.,.,2) is based on the manner
!>            other standard variables are managed in the same case.
!>            This type of boundary condition appears nonsense
!>            concerning mesh in that context.
!>
!>  - \c rcodcl(ifac,ivar,3) :
!>    Flux density (in kg/m s2) = J if icodcl(ifac, ivar) = 3
!>                 (<0 if gain, n outwards-facing normal)
!>    \c rcodcl(ifac,ivar,3) = -(VISCMA)* (grad Um).n
!>              (Um represents mesh velocity)
!>    \remark note that the definition of condition \c rcodcl(ifac,ivar,3)
!>            is based on the manner other standard variables are
!>            managed in the same case.
!>            \c rcodcl(.,.,3) = 0.d0 enables one to specify a homogeneous
!>            Neuman condition on mesh velocity. Any other value will be
!>            physically nonsense in that context.
!>
!> Note that if the user assigns a value to \c ialtyb equal to \c ibfixe, \c igliss,
!> or \c ivimpo and does not modify \c icodcl (zero value by
!> default), \c ialtyb will define the boundary condition type.
!>
!> To the contrary, if the user prescribes \c icodcl(ifac, ivar) (nonzero),
!> the values assigned to rcodcl will be used for the considered face
!> and variable (if rcodcl values are not set, the default values will
!> be used for the face and variable, so:
!>                         - \c rcodcl(ifac, ivar, 1) = 0.d0
!>                         - \c rcodcl(ifac, ivar, 2) = rinfin
!>                         - \c rcodcl(ifac, ivar, 3) = 0.d0)
!>
!> If the user decides to prescribe his own non-standard boundary conditions
!> it will be necessary to assign values to \c icodcl AND to rcodcl for ALL
!> mesh velocity components. Thus, the user does not need to assign values
!> to \c IALTYB for each associated face, as it will not be taken into account
!> in the code.
!>
!>
!> \subsection cons_rul_usa Consistency rules
!>
!> A consistency rules between \c icodcl codes for variables with
!> non-standard boundary conditions:
!>  - If a symmetry code (\c icodcl=4) is imposed for one mesh velocity
!>    component, one must have the same condition for all other mesh
!>    velocity components.
!>
!>
!> \subsection fix_nod_usa Fixed displacement on nodes
!>
!> For a better precision concerning mesh displacement, one can also assign values
!> of displacement to certain internal and/or boundary nodes. Thus, one
!> need to fill \c DEPALE and \c impale arrays :
!>  - \c disale(1,inod) = displacement of node inod in 'x' direction
!>  - \c disale(2,inod) = displacement of node inod in 'y' direction
!>  - \c disale(3,inod) = displacement of node inod in 'z' direction
!> This array is defined as the total displacement of the node compared
!> its initial position in initial mesh.
!> \c impale(inod) = 1 indicates that the displacement of node inod is imposed
!> \note Note that \c impale array is initialized to the value of 0; if its value
!>       is not modified, corresponding value in \c DEPALE array will not be
!>       taken into account
!>
!> During mesh's geometry re-calculation at each time step, the position of the
!> nodes, which displacement is fixed (i.e. \c impale=1), is not calculated
!> using the value of mesh velocity at the center of corresponding cell, but
!> directly filled using the values of \c DEPALE.
!>
!> If the displacement is fixed for all nodes of a boundary face it's not
!> necessary to prescribe boundary conditions at this face on mesh velocity.
!> \c icodcl and \c rcodcl values will be overwritten:
!>  - \c icodcl is automatically set to 1 (Dirichlet)
!>  - \c rcodcl value will be automatically set to face's mean mesh velocity
!>    value, that is calculated using \c DEPALE array.
!>
!> If a fixed boundary condition (\c ialtyb(ifac)=ibfixe) is imposed to the face
!> \c ifac, the displacement of each node inod belonging to ifac is considered
!> to be fixed, meaning that \c impale(inod) = 1 and \c disale(.,inod) = 0.d0.
!>
!>
!> \subsubsection nod_des_usa Description of nodes
!>
!> \c nnod gives the total (internal and boundary) number of nodes.
!> Vertices coordinates are given by \c xyznod(3, nnod) array. This table is
!> updated at each time step of the calculation.
!> \c xyzno0(3,nnod) gives the coordinates of initial mesh at the beginning
!> of the calculation.
!>
!> The faces - nodes connectivity is stored by means of four integer arrays :
!> \c ipnfac, \c nodfac, \c ipnfbr, \c nodfbr.
!>
!> \c nodfac(nodfbr) stores sequentially the index-numbers of the nodes of each
!> internal (boundary) face.
!> \c ipnfac(ipnfbr) gives the position of the first node of each internal
!> (boundary) face in the array \c nodfac(nodfbr).
!>
!> For example, in order to get all nodes of internal face \c ifac, one can
!> use the following loop:
!>
!> \code
!> do ii = ipnfac(ifac), ipnfac(ifac+1)-1 !! index number of nodfac array
!>                                        !! corresponding to ifac
!>
!>   inod = nodfac(ii)                    !! index-number iith node of face ifac.
!>   !! ...
!> enddo
!> \endcode
!>
!>
!> \subsection flui_bc Influence on boundary conditions related to fluid velocity
!>
!> The effect of fluid velocity and ALE modeling on boundary faces that
!> are declared as walls (\c itypfb = \c iparoi or \c iparug) really depends on
!> the physical nature of this interface.
!>
!> Indeed when studying an immersed structure the motion of corresponding
!> boundary faces is the one of the structure, meaning that it leads to
!> fluid motion. On the other hand when studying a piston the motion of vertices
!> belonging to lateral boundaries has no physical meaning therefore it has
!> no influence on fluid motion.
!>
!> Whatever the case, mesh velocity component that is normal to the boundary
!> face is always taken into account
!> (\f$ \vect{u}_{fluid} \cdot \vect{n} = \vect{w}_{mesh} \cdot \vect{n} \f$).
!>
!> The modeling of tangential mesh velocity component differs from one case
!> to another.
!>
!> The influence of mesh velocity on boundary conditions for fluid modeling is
!> managed and modeled in Code_Saturne as follows:
!>  - If \c ialtyb(ifac) = ibfixe: mesh velocity equals 0. (In case of 'fluid sliding
!>  wall' modeling corresponding condition will be specified in Code_Saturne
!>  Interface or in cs_user_boundary_conditions.f90 subroutine.)
!>  - If \c ialtyb(ifac) = ivimpo: tangential mesh velocity is modeled as a sliding
!>  wall velocity in fluid boundary conditions unless a value for fluid sliding
!>  wall velocity has been specified by USER in Code_Saturne Interface
!>  or in cs_user_boundary_conditions.f90 subroutine.
!>  - If \c ialtyb(ifac) = igliss: tangential mesh velocity is not taken into account
!>  in fluid boundary conditions (In case of 'fluid sliding wall' modeling
!>  corresponding condition will be specified in Code_Saturne Interface
!>  or in cs_user_boundary_conditions.f90 subroutine.)
!>  - If \c impale(inod) = 1 for all vertices of a boundary face: tangential mesh
!>  velocity value that has been derived from nodes displacement is modeled as a
!>  sliding wall velocity in fluid boundary conditions unless a value for fluid
!>  sliding wall velocity has been specified by USER in Code_Saturne Interface or
!>  in 'cs_user_boundary_conditions' subroutine.
!>
!> Note that mesh velocity has no influence on modeling of
!> boundary faces with 'inlet' or 'free outlet' fluid boundary condition.
!>
!> For "non standard" conditions USER has to manage the influence of boundary
!> conditions for ALE method (i.e. mesh velocity) on the ones for Navier Stokes
!> equations(i.e. fluid velocity). (Note that fluid boundary conditions can be
!> specified in this subroutine.)
!>
!>
!>\subsubsection cell_id_usa Cells identification
!>
!> Cells may be identified using the getcel subroutine.
!> The syntax of this subroutine is described in the
!> cs_user_boundary_conditions.f90 subroutine,
!> but a more thorough description can be found in the user guide.
!>
!>
!> \subsubsection fac_id_usa Faces identification
!>
!> Faces may be identified using the \ref getfbr subroutine.
!> The syntax of this subroutine is described in the
!> cs_user_boundary_conditions.f90 subroutine,
!> but a more thorough description can be found in the user guide.

!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]     itrale        number of iterations for ALE method
!> \param[in]     nvar          total number of variables
!> \param[in]     nscal         total number of scalars
!> \param[out]    icodcl        boundary condition code:
!>                               - 1 Dirichlet
!>                               - 2 Radiative outlet
!>                               - 3 Neumann
!>                               - 4 sliding and
!>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
!>                               - 5 smooth wall and
!>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
!>                               - 6 rough wall and
!>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
!>                               - 9 free inlet/outlet
!>                                 (input mass flux blocked to 0)
!> \param[in,out] itypfb        boundary face types
!> \param[out]    ialtyb        boundary face types for mesh velocity
!> \param[in]     impale        indicator for fixed node displacement
!> \param[in]     dt            time step (per cell)
!> \param[in,out] rcodcl        boundary condition values:
!>                               - rcodcl(1) value of the dirichlet
!>                               - rcodcl(2) value of the exterior exchange
!>                                 coefficient (infinite if no exchange)
!>                               - rcodcl(3) value flux density
!>                                 (negative if gain) in w/m2 or roughness
!>                                 in m if icodcl=6
!>                                 -# for the velocity \f$ (\mu+\mu_T)
!>                                    \gradv \vect{u} \cdot \vect{n}  \f$
!>                                 -# for the pressure \f$ \Delta t
!>                                    \grad P \cdot \vect{n}  \f$
!>                                 -# for a scalar \f$ cp \left( K +
!>                                     \dfrac{K_T}{\sigma_T} \right)
!>                                     \grad T \cdot \vect{n} \f$
!> \param[in,out] disale        nodes total displacement (from time 0 to n)
!> \param[in]     xyzno0        vertex coordinates of initial mesh
!_______________________________________________________________________________

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! File purpose: imposing/adapting ALE BC to the plates as well as defing the grid velocity
! Date:         30/07/18
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

subroutine usalcl &
 ( itrale ,                                                       &
   nvar   , nscal  ,                                              &
   icodcl , itypfb , ialtyb , impale ,                            &
   dt     ,                                                       &
   rcodcl , xyzno0 , disale )

!===============================================================================

!===============================================================================
! Module files
!===============================================================================

use paramx
use numvar
use optcal
use cstphy
use cstnum
use entsor
use parall
use period
use ihmpre
use mesh

! Module files added by the user
use field
use cs_c_bindings

!===============================================================================

implicit none
!< [arg]
! Arguments

integer          itrale
integer          nvar   , nscal

integer          icodcl(nfabor,nvar)
integer          itypfb(nfabor), ialtyb(nfabor)
integer          impale(nnod)
integer          iortvm

double precision dt(ncelet)
double precision rcodcl(nfabor,nvar,3)
double precision disale(3,nnod), xyzno0(3,nnod)

! double precision cvar_disale_prev(2,nfac)

double precision, dimension(:,:), pointer :: cvar_mvisc
double precision, dimension(:,:), pointer :: cvar_disale_prev
double precision, dimension(:,:), pointer :: cvar_mesh_v

!< [arg]
!< [loc_var]
! Local variables

integer          ifac, iel, i, ii , thres
integer          inod
integer          ilelt, nlelt
integer          ilcel, nlcel

integer, allocatable, dimension(:) :: lstelt
integer, allocatable, dimension(:) :: lstcel

! Local parameters for mesh viscosity
!------------------------------------
double precision viscmin , viscmax
!-----------------------------------

! Declaration of the parameters of the forcing velocity term
!-----------------------------------------------------------
double precision f, w, hn, a
double precision delta0
!-----------------------------------------------------------

! Declaration of the length of the hexagon edges
!-----------------------------------------------
double precision L , D , semiP0
!-----------------------------------------------

! Declaration of the parameters for azimuts
!------------------------------------------
double precision theta_NE
double precision theta_IA_SE , theta_IA_NE , theta_IA_NW , theta_IA_SW
double precision theta_SE_comp , theta_NE_comp , theta_NW_comp , theta_SW_comp
!------------------------------------------

! Declaration of the parameters for displacement
!-----------------------------------------------------------
double precision depl_O(2)  , depl_S(2) , depl_SE(2)
double precision depl_NE(2) , depl_N(2) , depl_NW(2) , depl_SW(2)
!-----------------------------------------------------------

! Nodes' coordinates with respect to the reference axis system
!-------------------------------------------------------------
double precision xO_S  , yO_S
double precision xO_NE , yO_NE
double precision xO_N  , yO_N
double precision xO_NW , yO_NW
double precision xO_SW , yO_SW
double precision xO_SE , yO_SE
!-------------------------------------------------------------

! Nodes' coordinates with respect to the reference axis system
!-------------------------------------------------------------
double precision xO_SE_comp , yO_SE_comp
double precision xO_E_comp  , yO_E_comp
double precision xO_NE_comp , yO_NE_comp
double precision xO_NW_comp , yO_NW_comp
double precision xO_W_comp  , yO_W_comp
double precision xO_SW_comp , yO_SW_comp
!-------------------------------------------------------------

! Nodes' coordinates with respect to the local axis system
!---------------------------------------------------------
double precision x_SE , y_SE
double precision x_NE , y_NE 
double precision x_N  , y_N
double precision x_NW , y_NW 
double precision x_SW , y_SW
double precision x_S  , y_S
!---------------------------------------------------------

! Nodes' coordinates with respect to the local axis system
!---------------------------------------------------------
double precision x_SE_comp , y_SE_comp
double precision x_E_comp  , y_E_comp
double precision x_NE_comp , y_NE_comp
double precision x_NW_comp , y_NW_comp
double precision x_W_comp  , y_W_comp
double precision x_SW_comp , y_SW_comp
!-------------------------------------------------------------

! Shape functions for the triangular zone
!----------------------------------------
double precision sf1_SE , sf2_SE , sf3_SE
double precision sf1_E  , sf2_E  , sf3_E
double precision sf1_NE , sf2_NE , sf3_NE
double precision sf1_NW , sf2_NW , sf3_NW
double precision sf1_W  , sf2_W  , sf3_W
double precision sf1_SW , sf2_SW , sf3_SW
!----------------------------------------

!< [loc_var]
!===============================================================================

!===============================================================================
! 1.  Initialization
!===============================================================================
!< [allocate_ale]
! Allocate a temporary array for boundary faces selection
allocate(lstelt(nfabor))
! Allocate a temporary array for cells selection
allocate(lstcel(ncel))
!< [allocate_ale]
!===============================================================================

! Setup of the parameters of the forcing velocity term
!-----------------------------------------------------
f   = 20d0
! f   = 10d0 ! Fréquence maximale PISE-1A

w   = 2*pi*f
hn  = 3.0d-3
a   = -0.5d0  
!-----------------------------------------------------

! Additional geometrical parameter (hexagon edge)
L = 0.07332d0
D = (sqrt(3.d0)/2.d0)*L

! Initial gap
delta0    = hn*(1+a)

semiP0 = D + delta0/2.d0

! Orientation

! Localisation polaire
theta_NE      = pi/6

! Orientation des repères
theta_IA_SE   =   pi/3
theta_IA_NE   = 2*pi/3
theta_IA_NW   = 4*pi/3
theta_IA_SW   = 5*pi/3

theta_SE_comp =  -pi/3
theta_NE_comp =   pi/3
theta_NW_comp = 2*pi/3
theta_SW_comp = 4*pi/3

! Spatial coordinates of the inter-spaces center (radial)
xO_NE_comp = (1.5d0*L + sqrt(3.d0)/2.d0*delta0)*cos(theta_NE_comp) 
yO_NE_comp = (1.5d0*L + sqrt(3.d0)/2.d0*delta0)*sin(theta_NE_comp)  

! Spatial coordinates of the inter-spaces center (orthoradial)
xO_NE = semiP0*cos(theta_NE)
yO_NE = semiP0*sin(theta_NE)  

!< [calcul]

!===============================================================================

! Mesh viscosity
! iortvm = 0 ! Isotropic
! iortvm = 1 ! Orthotropic

!#if 0
! ################################################################################################################
! ############################################ ALE BOUNDARY CONDITIONS ###########################################
! ################################################################################################################

#if 0
! ######################################################################
! ############################### INLETS ###############################
! ######################################################################

call getfbr('IAcomp_fC_inlet_right_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)
    
  ! ialtyb(ifac) = ivimpo
  ! ialtyb(ifac) = ibfixe
  
  icodcl(ifac, iuma) = 3
  icodcl(ifac, ivma) = 3
  icodcl(ifac, iwma) = 3
  
  rcodcl(ifac, iuma, 3) = 0.d0
  rcodcl(ifac, ivma, 3) = 0.d0
  rcodcl(ifac, iwma, 3) = 0.d0
  
enddo

! ######################################################################
! ######################################################################
#endif

! ######################################################################
! ########################## HEXAGONES EDGES ########################### 
! ######################################################################
!#if 0
call getfbr('IA_fC_S_top_extruded or IA_fC_S_bottom_extruded or IA_fC_hp_SE_top_extruded&
& or IA_fC_hp_SE_bottom_extruded or IA_fC_SE_top_extruded or IA_fC_SE_bottom_extruded&
& or IA_fC_hp_E_top_extruded or IA_fC_hp_E_bottom_extruded or IA_fC_NE_top_extruded&
& or IA_fC_NE_bottom_extruded or IA_fC_hp_NE_top_extruded or IA_fC_hp_NE_bottom_extruded&
& or IA_fC_N_top_extruded or IA_fC_N_bottom_extruded or IA_fC_hp_NW_top_extruded&
& or IA_fC_hp_NW_bottom_extruded or IA_fC_NW_top_extruded or IA_fC_NW_bottom_extruded&
& or IA_fC_hp_W_top_extruded or IA_fC_hp_W_bottom_extruded or IA_fC_SW_top_extruded&
& or IA_fC_SW_bottom_extruded or IA_fC_hp_SW_top_extruded or IA_fC_hp_SW_bottom_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)
    
  ialtyb(ifac) = ibfixe
  ! ialtyb(ifac) = igliss
enddo
!#endif
! ######################################################################
! ######################################################################

#if 0
! ######################################################################
! ########################## TOP/BOTTOM FACES ##########################
! ######################################################################

call getfbr('IA_fC_SE_triangle or IA_fC_E_triangle or IA_fC_NE_triangle&
& or IA_fC_NW_triangle or IA_fC_W_triangle or IA_fC_SW_triangle or IA_fC_S_O_plate&
& or IA_fC_SE_O_plate or IA_fC_NE_O_plate or IA_fC_N_O_plate&
& or IA_fC_NW_O_plate or IA_fC_SW_O_plate&
& or IA_fC_SE_SE_plate or IA_fC_E_SE_plate or IA_fC_NE_SE_plate or IA_fC_NW_SE_plate&
& or IA_fC_W_SE_plate or IA_fC_SW_SE_plate', nlelt, lstelt)

do ilelt = 1, nlelt
  
  ifac  = lstelt(ilelt)
 
  ialtyb(ifac) = igliss
  
enddo 

call getfbr('IA_fC_SE_triangle_top or IA_fC_E_triangle_top or IA_fC_NE_triangle_top&
& or IA_fC_NW_triangle_top or IA_fC_W_triangle_top or IA_fC_SW_triangle_top&
& or IA_fC_S_O_plate_top or IA_fC_SE_O_plate_top& 
& or IA_fC_NE_O_plate_top or IA_fC_N_O_plate_top&
& or IA_fC_NW_O_plate_top or IA_fC_SW_O_plate_top or IA_fC_SE_SE_plate_top&
& or IA_fC_E_SE_plate_top or IA_fC_NE_SE_plate_top or IA_fC_NW_SE_plate_top&
& or IA_fC_W_SE_plate_top or IA_fC_SW_SE_plate_top', nlelt, lstelt)

do ilelt = 1, nlelt
  
  ifac  = lstelt(ilelt)
 
  ialtyb(ifac) = igliss
  
enddo 

! ######################################################################
! ######################################################################
#endif

! Call to nodes displacement field
call field_get_val_prev_v_by_name("disale",cvar_disale_prev)
call field_get_val_prev_v(ivarfl(iuma), cvar_mesh_v)
 
thres = 0;
!===============================================================================

! Setting the velocity profile to the top/bottom plates
!#if 0
! ######################################################################
! ########################## HEXAGONES EDGES ########################### 
! ######################################################################

! %%%%%%%%%%%%%%%%%%%%%%%%%% CENTRAL HEXAGONE %%%%%%%%%%%%%%%%%%%%%%%%%%
 
call getfbr('IA_fC_S_top_extruded or IA_fC_SE_top_extruded or IA_fC_NE_top_extruded&
& or IA_fC_N_top_extruded or IA_fC_NW_top_extruded or IA_fC_SW_top_extruded', nlelt, lstelt)

ifac  = lstelt(1)
i = ipnfbr(ifac)
inod = nodfbr(i)

! if ((ntcabs .eq. 0) .or. (ntcabs .eq. 1)) then
if (ntcabs .le. thres) then  
  depl_O(1) = 0.d0
  depl_O(2) = 0.d0
else
  depl_O(1) = cvar_disale_prev(1,inod)
  depl_O(2) = cvar_disale_prev(2,inod)
  
  !depl_O(1) = cvar_mesh_v(1,ifac)*dtref
  !depl_O(2) = cvar_mesh_v(2,ifac)*dtref
endif
  
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! %%%%%%%%%%%%%%%%%%%%%%%%% SOUTHERN HEXAGONE %%%%%%%%%%%%%%%%%%%%%%%%%%

call getfbr('IA_fC_hp_SW_top_extruded or IA_fC_S_bottom_extruded&
& or IA_fC_hp_SE_bottom_extruded', nlelt, lstelt)

ifac  = lstelt(1)
i = ipnfbr(ifac)
inod = nodfbr(i)

! if ((ntcabs .eq. 0) .or. (ntcabs .eq. 1)) then
if (ntcabs .le. thres) then  
  depl_S(1) = 0.d0
  depl_S(2) = 0.d0
else
  depl_S(1) = cvar_disale_prev(1,inod)
  depl_S(2) = cvar_disale_prev(2,inod)
  
  !depl_S(1) = cvar_mesh_v(1,ifac)*dtref
  !depl_S(2) = cvar_mesh_v(2,ifac)*dtref
endif
  
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! %%%%%%%%%%%%%%%%%%%%%%% SOUTHEASTERN HEXAGONE %%%%%%%%%%%%%%%%%%%%%%%%

call getfbr('IA_fC_hp_SE_top_extruded or IA_fC_SE_bottom_extruded&
& or IA_fC_hp_E_bottom_extruded', nlelt, lstelt)

ifac  = lstelt(1)
i = ipnfbr(ifac)
inod = nodfbr(i)

! if ((ntcabs .eq. 0) .or. (ntcabs .eq. 1)) then
if (ntcabs .le. thres) then  
  depl_SE(1) = 0.d0
  depl_SE(2) = 0.d0
else
  depl_SE(1) = cvar_disale_prev(1,inod)
  depl_SE(2) = cvar_disale_prev(2,inod)
  
  !depl_SE(1) = cvar_mesh_v(1,ifac)*dtref
  !depl_SE(2) = cvar_mesh_v(2,ifac)*dtref
endif
  
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! %%%%%%%%%%%%%%%%%%%%%%% NORTHEASTERN HEXAGONE %%%%%%%%%%%%%%%%%%%%%%%%

call getfbr('IA_fC_hp_E_top_extruded or IA_fC_NE_bottom_extruded&
& or IA_fC_hp_NE_bottom_extruded', nlelt, lstelt)

ifac  = lstelt(1)
i = ipnfbr(ifac)
inod = nodfbr(i)

! if ((ntcabs .eq. 0) .or. (ntcabs .eq. 1)) then
if (ntcabs .le. thres) then  
  depl_NE(1) = 0.d0
  depl_NE(2) = 0.d0
else
  depl_NE(1) = cvar_disale_prev(1,inod)
  depl_NE(2) = cvar_disale_prev(2,inod)
  
  !depl_NE(1) = cvar_mesh_v(1,ifac)*dtref
  !depl_NE(2) = cvar_mesh_v(2,ifac)*dtref
endif
  
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! %%%%%%%%%%%%%%%%%%%%%%%%% NORTHERN HEXAGONE %%%%%%%%%%%%%%%%%%%%%%%%%%

call getfbr('IA_fC_hp_NE_top_extruded or IA_fC_N_bottom_extruded&
& or IA_fC_hp_NW_bottom_extruded', nlelt, lstelt)

ifac  = lstelt(1)
i = ipnfbr(ifac)
inod = nodfbr(i)

! if ((ntcabs .eq. 0) .or. (ntcabs .eq. 1)) then
if (ntcabs .le. thres) then  
  depl_N(1) = 0.d0
  depl_N(2) = 0.d0
else
  depl_N(1) = cvar_disale_prev(1,inod)
  depl_N(2) = cvar_disale_prev(2,inod)
  
  !depl_N(1) = cvar_mesh_v(1,ifac)*dtref
  !depl_N(2) = cvar_mesh_v(2,ifac)*dtref
endif
  
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! %%%%%%%%%%%%%%%%%%%%%%% NORTHWESTERN HEXAGONE %%%%%%%%%%%%%%%%%%%%%%%%

call getfbr('IA_fC_hp_NW_top_extruded or IA_fC_NW_bottom_extruded&
& or IA_fC_hp_W_bottom_extruded', nlelt, lstelt)

ifac  = lstelt(1)
i = ipnfbr(ifac)
inod = nodfbr(i)

! if ((ntcabs .eq. 0) .or. (ntcabs .eq. 1)) then
if (ntcabs .le. thres) then  
  depl_NW(1) = 0.d0
  depl_NW(2) = 0.d0
else
  depl_NW(1) = cvar_disale_prev(1,inod)
  depl_NW(2) = cvar_disale_prev(2,inod)
  
  !depl_NW(1) = cvar_mesh_v(1,ifac)*dtref
  !depl_NW(2) = cvar_mesh_v(2,ifac)*dtref
endif
  
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! %%%%%%%%%%%%%%%%%%%%%%% SOUTHWESTERN HEXAGONE %%%%%%%%%%%%%%%%%%%%%%%%

call getfbr('IA_fC_hp_W_top_extruded or IA_fC_SW_bottom_extruded&
& or IA_fC_hp_SW_bottom_extruded', nlelt, lstelt)

ifac  = lstelt(1)
i = ipnfbr(ifac)
inod = nodfbr(i)

! if ((ntcabs .eq. 0) .or. (ntcabs .eq. 1)) then
if (ntcabs .le. thres) then  
  depl_SW(1) = 0.d0
  depl_SW(2) = 0.d0
else
  depl_SW(1) = cvar_disale_prev(1,inod)
  depl_SW(2) = cvar_disale_prev(2,inod)
  
  !depl_SW(1) = cvar_mesh_v(1,ifac)*dtref
  !depl_SW(2) = cvar_mesh_v(2,ifac)*dtref
endif
  
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!#endif

! ################################################################################################################
! ################################################################################################################
!#endif

! ######################################################################
! ########################### MESH VISCOSITY ###########################
! ######################################################################

#if 0
call field_get_val_v_by_name("mesh_viscosity",cvar_mvisc)
call getcel('all[]', nlcel, lstcel)

do ilelt = 1, ncel

  iel  = lstcel(ilelt)
  
  cvar_mvisc(1,iel) = 1.d1
  cvar_mvisc(2,iel) = 1.d1
  cvar_mvisc(3,iel) = 1.d1
  
enddo
#endif
  
! ######################################################################
! ############################# TRIANGLES ##############################
! ######################################################################

! call field_get_val_v_by_name("mesh_viscosity",cvar_mvisc)

#if 0
! ****************************** TRIANGLE ******************************
call getcel('IA_fC_SE_triangle or IA_fC_SE_triangle_top', nlcel, lstcel)

do ilelt = 1, ncel

  iel  = lstcel(ilelt)
    
  sf1_SE = 1 - 2/(sqrt(3.d0)*delta0)*(0.5*(L+sqrt(3.d0)*delta0) - xyzcen(1,iel))
  sf2_SE =     1/(sqrt(3.d0)*delta0)*(0.5*(L+sqrt(3.d0)*delta0) - xyzcen(1,iel)&
   + sqrt(3.d0)*(xyzcen(2,iel) + (D + delta0/2.d0)))
  sf3_SE =     1/(sqrt(3.d0)*delta0)*(0.5*(L+sqrt(3.d0)*delta0) - xyzcen(1,iel)&
   - sqrt(3.d0)*(xyzcen(2,iel) + (D + delta0/2.d0)))
  
  viscmax = 1.0d10
  viscmin = 1.0d0
    
  cvar_mvisc(1,iel) =  viscmax + (viscmin - viscmax)  * (sf1_SE * sf2_SE * sf3_SE)
  cvar_mvisc(2,iel) =  viscmax + (viscmin - viscmax)  * (sf1_SE * sf2_SE * sf3_SE)
  cvar_mvisc(3,iel) =  viscmax + (viscmin - viscmax)  * (sf1_SE * sf2_SE * sf3_SE)
    
enddo 
! **********************************************************************
#endif

#if 0
! ****************************** TRIANGLE ******************************
call getfbr('IA_fC_E_triangle or IA_fC_E_triangle_top', nlelt, lstelt)

do ilelt = 1, ncel

  iel  = lstcel(ilelt)
    
  sf1_E = -1/(sqrt(3.d0)*delta0)*( (L- xyzno0(1,inod)) - sqrt(3.d0)*xyzno0(2,inod) )
  
  sf3_E = -1/(sqrt(3.d0)*delta0)*( (L- xyzno0(1,inod)) + sqrt(3.d0)*xyzno0(2,inod) )
  
  viscmax = 1.0d10
  viscmin = 1.0d0
    
  cvar_mvisc(1,iel) =  viscmax + (viscmin - viscmax)  * (sf1_SE * sf2_SE * sf3_SE)
  cvar_mvisc(2,iel) =  viscmax + (viscmin - viscmax)  * (sf1_SE * sf2_SE * sf3_SE)
  cvar_mvisc(3,iel) =  viscmax + (viscmin - viscmax)  * (sf1_SE * sf2_SE * sf3_SE)
    
enddo 
! **********************************************************************
#endif

! ################################################################################################################
! ########################################## GRID VELOCITY INTERPOLATION #########################################
! ################################################################################################################
!#if 0
! %%%%%%%%%%%%%%%%%%%%%%%%%%% SOUTHERN ZONE %%%%%%%%%%%%%%%%%%%%%%%%%%%%

call getfbr('IA_fC_S_O_plate or IA_fC_S_O_plate_top', nlelt, lstelt)

yO_S = -semiP0

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)
  
  do ii = ipnfbr(ifac), ipnfbr(ifac+1)-1
  
    inod = nodfbr(ii)
    
    if (impale(inod).eq.0) then
    
      y_S = xyzno0(2,inod) - yO_S
      
      disale(1, inod) = (depl_O(1) - depl_S(1))/delta0 * y_S + (depl_O(1) + depl_S(1))/2.d0
      disale(2, inod) = (depl_O(2) - depl_S(2))/delta0 * y_S + (depl_O(2) + depl_S(2))/2.d0
      disale(3, inod) = 0.d0
      
      impale(inod) = 1
      
    endif
    
  enddo 
  
enddo

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

!#if 0
! %%%%%%%%%%%%%%%%%%%%% SOUTHEASTERN ZONE (RADIAL) %%%%%%%%%%%%%%%%%%%%%

! ****************************** TRIANGLE ******************************
call getfbr('IA_fC_SE_triangle or IA_fC_SE_triangle_top', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)

  do ii = ipnfbr(ifac), ipnfbr(ifac+1)-1
  
    inod = nodfbr(ii)
    
    sf1_SE = 1 - 2/(sqrt(3.d0)*delta0)*(0.5*(L+sqrt(3.d0)*delta0) - xyzno0(1,inod))
    sf3_SE =     1/(sqrt(3.d0)*delta0)*(0.5*(L+sqrt(3.d0)*delta0) - xyzno0(1,inod)&
     - sqrt(3.d0)*(xyzno0(2,inod) + (D + delta0/2.d0)))
    sf2_SE =  1 - (sf1_SE + sf3_SE)

    if (impale(inod).eq.0) then
    
      disale(1, inod) = depl_SE(1) * sf1_SE + depl_O(1) * sf2_SE + depl_S(1) * sf3_SE
      disale(2, inod) = depl_SE(2) * sf1_SE + depl_O(2) * sf2_SE + depl_S(2) * sf3_SE
      disale(3, inod) = 0.d0
      
      impale(inod) = 1
      
    endif
    
  enddo 
  
enddo
! **********************************************************************

! ******************************* PLATE ******************************** 
call getfbr('IA_fC_SE_SE_plate or IA_fC_SE_SE_plate_top or IAcomp_fC_SE_inlet_right_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)
   
  do ii = ipnfbr(ifac), ipnfbr(ifac+1)-1
  
    inod = nodfbr(ii)
    
    if (impale(inod).eq.0) then
    
      y_SE_comp = -sin(theta_SE_comp)*(xyzno0(1,inod) - xO_NE_comp) + cos(theta_SE_comp)*(xyzno0(2,inod) - (-yO_NE_comp))
   
      disale(1, inod) = (depl_SE(1) - depl_S(1))/delta0 * y_SE_comp + (depl_SE(1) + depl_S(1))/2.d0
      disale(2, inod) = (depl_SE(2) - depl_S(2))/delta0 * y_SE_comp + (depl_SE(2) + depl_S(2))/2.d0
      disale(3, inod) = 0.d0
      
      impale(inod) = 1
      
    endif
    
  enddo 
  
enddo
! **********************************************************************

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! %%%%%%%%%%%%%%%%%% SOUTHEASTERN ZONE (ORTHORADIAL) %%%%%%%%%%%%%%%%%%%

call getfbr('IA_fC_SE_O_plate or IA_fC_SE_O_plate_top', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)
   
  do ii = ipnfbr(ifac), ipnfbr(ifac+1)-1
  
    inod = nodfbr(ii)
    
    if (impale(inod).eq.0) then
      
      y_SE = -sin(theta_IA_SE)*(xyzno0(1,inod) - xO_NE) + cos(theta_IA_SE)*(xyzno0(2,inod) - (-yO_NE))
      
      disale(1, inod) = (depl_O(1) - depl_SE(1))/delta0 * y_SE + (depl_O(1) + depl_SE(1))/2.d0
      disale(2, inod) = (depl_O(2) - depl_SE(2))/delta0 * y_SE + (depl_O(2) + depl_SE(2))/2.d0
      disale(3, inod) = 0.d0
      
      impale(inod) = 1
      
    endif
    
  enddo 
!#endif 
  
enddo

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! %%%%%%%%%%%%%%%%%%%%%%% EASTERN ZONE (RADIAL) %%%%%%%%%%%%%%%%%%%%%%%%

! ****************************** TRIANGLE ******************************
call getfbr('IA_fC_E_triangle or IA_fC_E_triangle_top', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)

  do ii = ipnfbr(ifac), ipnfbr(ifac+1)-1
  
    inod = nodfbr(ii)
    
    sf1_E = -1/(sqrt(3.d0)*delta0)*( (L- xyzno0(1,inod)) - sqrt(3.d0)*xyzno0(2,inod) )
    sf3_E = -1/(sqrt(3.d0)*delta0)*( (L- xyzno0(1,inod)) + sqrt(3.d0)*xyzno0(2,inod) )
    sf2_E =  1 - (sf1_E + sf3_E)
    
    if (impale(inod).eq.0) then
    
      disale(1, inod) = depl_NE(1) * sf1_E + depl_O(1) * sf2_E + depl_SE(1) * sf3_E
      disale(2, inod) = depl_NE(2) * sf1_E + depl_O(2) * sf2_E + depl_SE(2) * sf3_E
      disale(3, inod) = 0.d0
      
      impale(inod) = 1
      
    endif
    
  enddo 
  
enddo
! **********************************************************************

! ******************************* PLATE ******************************** 
call getfbr('IA_fC_E_SE_plate or IA_fC_E_SE_plate_top or IAcomp_fC_E_inlet_right_extruded', nlelt, lstelt)

yO_E_comp = 0.d0

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)
   
  do ii = ipnfbr(ifac), ipnfbr(ifac+1)-1
  
    inod = nodfbr(ii)
    
    if (impale(inod).eq.0) then
    
      y_E_comp = xyzno0(2,inod) - yO_E_comp
  
      disale(1, inod) = (depl_NE(1) - depl_SE(1))/delta0 * y_E_comp + (depl_NE(1) + depl_SE(1))/2.d0
      disale(2, inod) = (depl_NE(2) - depl_SE(2))/delta0 * y_E_comp + (depl_NE(2) + depl_SE(2))/2.d0
      disale(3, inod) = 0.d0
      
      impale(inod) = 1
      
    endif
    
  enddo 
  
enddo
! **********************************************************************

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! %%%%%%%%%%%%%%%%%% NORTHEASTERN ZONE (ORTHORADIAL) %%%%%%%%%%%%%%%%%%%

call getfbr('IA_fC_NE_O_plate or IA_fC_NE_O_plate_top', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)
   
  do ii = ipnfbr(ifac), ipnfbr(ifac+1)-1
  
    inod = nodfbr(ii)
    
    if (impale(inod).eq.0) then
      
      y_NE = -sin(theta_IA_NE)*(xyzno0(1,inod) - xO_NE) + cos(theta_IA_NE)*(xyzno0(2,inod) - yO_NE)
      
      disale(1, inod) = (depl_O(1) - depl_NE(1))/delta0 * y_NE + (depl_O(1) + depl_NE(1))/2.d0
      disale(2, inod) = (depl_O(2) - depl_NE(2))/delta0 * y_NE + (depl_O(2) + depl_NE(2))/2.d0
      disale(3, inod) = 0.d0
      
      impale(inod) = 1
      
    endif
    
  enddo 
  
enddo

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! %%%%%%%%%%%%%%%%%%%% NORTHEASTERN ZONE (RADIAL) %%%%%%%%%%%%%%%%%%%%%%

! ****************************** TRIANGLE ******************************
call getfbr('IA_fC_NE_triangle or IA_fC_NE_triangle_top', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)

  do ii = ipnfbr(ifac), ipnfbr(ifac+1)-1
  
    inod = nodfbr(ii)
    
    sf1_NE =     1/(sqrt(3.d0)*delta0)*(0.5*(L+sqrt(3.d0)*delta0) - xyzno0(1,inod)&
     - sqrt(3.d0)*(-xyzno0(2,inod) + (D + delta0/2.d0)))
    sf3_NE = 1 - 2/(sqrt(3.d0)*delta0)*(0.5*(L+sqrt(3.d0)*delta0) - xyzno0(1,inod))
    sf2_NE =  1 - (sf1_NE + sf3_NE)
    
    if (impale(inod).eq.0) then
    
      disale(1, inod) = depl_N(1) * sf1_NE + depl_O(1) * sf2_NE + depl_NE(1) * sf3_NE
      disale(2, inod) = depl_N(2) * sf1_NE + depl_O(2) * sf2_NE + depl_NE(2) * sf3_NE
      disale(3, inod) = 0.d0
      
      impale(inod) = 1
      
    endif
    
  enddo 
  
enddo
! **********************************************************************

! ******************************* PLATE ******************************** 
call getfbr('IA_fC_NE_SE_plate or IA_fC_NE_SE_plate_top or IAcomp_fC_NE_inlet_right_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)
   
  do ii = ipnfbr(ifac), ipnfbr(ifac+1)-1
  
    inod = nodfbr(ii)
    
    if (impale(inod).eq.0) then
  
      y_NE_comp = -sin(theta_NE_comp)*(xyzno0(1,inod) - xO_NE_comp) + cos(theta_NE_comp)*(xyzno0(2,inod) - yO_NE_comp)
    
      disale(1, inod) = (depl_N(1) - depl_NE(1))/delta0 * y_NE_comp + (depl_N(1) + depl_NE(1))/2.d0
      disale(2, inod) = (depl_N(2) - depl_NE(2))/delta0 * y_NE_comp + (depl_N(2) + depl_NE(2))/2.d0
      disale(3, inod) = 0.d0
      
      impale(inod) = 1
      
    endif
    
  enddo 
  
enddo
! **********************************************************************

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! %%%%%%%%%%%%%%%%%%%%%%%%%%% NORTHERN ZONE %%%%%%%%%%%%%%%%%%%%%%%%%%%%

call getfbr('IA_fC_N_O_plate or IA_fC_N_O_plate_top', nlelt, lstelt)

yO_N = semiP0

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)
  
  do ii = ipnfbr(ifac), ipnfbr(ifac+1)-1
  
    inod = nodfbr(ii)
    
    if (impale(inod).eq.0) then
    
      ! Rotation of pi rad angle -> inversed coordinate value
      y_N = -(xyzno0(2,inod) - yO_N)
    
      disale(1, inod) = (depl_O(1) - depl_N(1))/delta0 * y_N + (depl_O(1) + depl_N(1))/2.d0
      disale(2, inod) = (depl_O(2) - depl_N(2))/delta0 * y_N + (depl_O(2) + depl_N(2))/2.d0
      disale(3, inod) = 0.d0
      
      impale(inod) = 1
      
    endif
    
  enddo 
  
enddo

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! %%%%%%%%%%%%%%%%%%%% NORTHWESTERN ZONE (RADIAL) %%%%%%%%%%%%%%%%%%%%%%

! ****************************** TRIANGLE ******************************
call getfbr('IA_fC_NW_triangle or IA_fC_NW_triangle_top', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)

  do ii = ipnfbr(ifac), ipnfbr(ifac+1)-1
  
    inod = nodfbr(ii)
    
    sf1_NW = 1 - 2/(sqrt(3.d0)*delta0)*(0.5*(L+sqrt(3.d0)*delta0) + xyzno0(1,inod))
    sf3_NW =     1/(sqrt(3.d0)*delta0)*(0.5*(L+sqrt(3.d0)*delta0) + xyzno0(1,inod)&
     - sqrt(3.d0)*(-xyzno0(2,inod) + (D + delta0/2.d0)))
    sf2_NW =  1 - (sf1_NW + sf3_NW)
    
    if (impale(inod).eq.0) then
    
      disale(1, inod) = depl_NW(1) * sf1_NW + depl_O(1) * sf2_NW + depl_N(1) * sf3_NW
      disale(2, inod) = depl_NW(2) * sf1_NW + depl_O(2) * sf2_NW + depl_N(2) * sf3_NW
      disale(3, inod) = 0.d0
      
      impale(inod) = 1
      
    endif
    
  enddo 
  
enddo
! **********************************************************************

! ******************************* PLATE ******************************** 
call getfbr('IA_fC_NW_SE_plate or IA_fC_NW_SE_plate_top or IAcomp_fC_NW_inlet_right_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)
   
  do ii = ipnfbr(ifac), ipnfbr(ifac+1)-1
  
    inod = nodfbr(ii)
    
    if (impale(inod).eq.0) then
    
      y_NW_comp = -sin(theta_NW_comp)*(xyzno0(1,inod) - (-xO_NE_comp)) + cos(theta_NW_comp)*(xyzno0(2,inod) - yO_NE_comp)
      
      disale(1, inod) = (depl_NW(1) - depl_N(1))/delta0 * y_NW_comp + (depl_NW(1) + depl_N(1))/2.d0
      disale(2, inod) = (depl_NW(2) - depl_N(2))/delta0 * y_NW_comp + (depl_NW(2) + depl_N(2))/2.d0
      disale(3, inod) = 0.d0
      
      impale(inod) = 1
      
    endif
    
  enddo 
  
enddo
! **********************************************************************

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! %%%%%%%%%%%%%%%%%% NORTHWESTERN ZONE (ORTHORADIAL) %%%%%%%%%%%%%%%%%%%

call getfbr('IA_fC_NW_O_plate or IA_fC_NW_O_plate_top', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)
   
  do ii = ipnfbr(ifac), ipnfbr(ifac+1)-1
  
    inod = nodfbr(ii)
    
    if (impale(inod).eq.0) then
      
      y_NW = -sin(theta_IA_NW)*(xyzno0(1,inod) - (-xO_NE)) + cos(theta_IA_NW)*(xyzno0(2,inod) - yO_NE)
      
      disale(1, inod) = (depl_O(1) - depl_NW(1))/delta0 * y_NW + (depl_O(1) + depl_NW(1))/2.d0
      disale(2, inod) = (depl_O(2) - depl_NW(2))/delta0 * y_NW + (depl_O(2) + depl_NW(2))/2.d0
      disale(3, inod) = 0.d0
      
      impale(inod) = 1
      
    endif
    
  enddo 
!#endif 
  
enddo

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! %%%%%%%%%%%%%%%%%%%%%%% WESTERN ZONE (RADIAL) %%%%%%%%%%%%%%%%%%%%%%%%

! ****************************** TRIANGLE ******************************
call getfbr('IA_fC_W_triangle or IA_fC_W_triangle_top', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)

  do ii = ipnfbr(ifac), ipnfbr(ifac+1)-1
  
    inod = nodfbr(ii)
    
    sf1_W = -1/(sqrt(3.d0)*delta0)*( (L + xyzno0(1,inod)) + sqrt(3.d0)*xyzno0(2,inod) )
    sf3_W = -1/(sqrt(3.d0)*delta0)*( (L + xyzno0(1,inod)) - sqrt(3.d0)*xyzno0(2,inod) )
    sf2_W =  1 - (sf1_W + sf3_W)

    if (impale(inod).eq.0) then
    
      disale(1, inod) = depl_SW(1) * sf1_W + depl_O(1) * sf2_W + depl_NW(1) * sf3_W
      disale(2, inod) = depl_SW(2) * sf1_W + depl_O(2) * sf2_W + depl_NW(2) * sf3_W
      disale(3, inod) = 0.d0
      
      impale(inod) = 1
      
    endif
    
  enddo 
  
enddo
! **********************************************************************

! ******************************* PLATE ******************************** 
call getfbr('IA_fC_W_SE_plate or IA_fC_W_SE_plate_top or IAcomp_fC_W_inlet_right_extruded', nlelt, lstelt)

yO_W_comp = 0.d0

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)
   
  do ii = ipnfbr(ifac), ipnfbr(ifac+1)-1
  
    inod = nodfbr(ii)
    
    if (impale(inod).eq.0) then
      
      y_W_comp = -(xyzno0(2,inod) - yO_W_comp)
      
      disale(1, inod) = (depl_SW(1) - depl_NW(1))/delta0 * y_W_comp + (depl_SW(1) + depl_NW(1))/2.d0
      disale(2, inod) = (depl_SW(2) - depl_NW(2))/delta0 * y_W_comp + (depl_SW(2) + depl_NW(2))/2.d0
      disale(3, inod) = 0.d0
      
      impale(inod) = 1
      
    endif
    
  enddo 
  
enddo
! **********************************************************************

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! %%%%%%%%%%%%%%%%%% SOUTHWESTERN ZONE (ORTHORADIAL) %%%%%%%%%%%%%%%%%%%

call getfbr('IA_fC_SW_O_plate or IA_fC_SW_O_plate_top', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)
   
  do ii = ipnfbr(ifac), ipnfbr(ifac+1)-1
  
    inod = nodfbr(ii)
    
    if (impale(inod).eq.0) then
      
      y_SW = -sin(theta_IA_SW)*(xyzno0(1,inod) - (-xO_NE)) + cos(theta_IA_SW)*(xyzno0(2,inod) - (-yO_NE))
      
      disale(1, inod) = (depl_O(1) - depl_SW(1))/delta0 * y_SW + (depl_O(1) + depl_SW(1))/2.d0
      disale(2, inod) = (depl_O(2) - depl_SW(2))/delta0 * y_SW + (depl_O(2) + depl_SW(2))/2.d0
      disale(3, inod) = 0.d0
      
      impale(inod) = 1
      
    endif
    
  enddo 
  
enddo

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! %%%%%%%%%%%%%%%%%%%% SOUTHWESTERN ZONE (RADIAL) %%%%%%%%%%%%%%%%%%%%%%

! ****************************** TRIANGLE ******************************
call getfbr('IA_fC_SW_triangle or IA_fC_SW_triangle_top', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)

  do ii = ipnfbr(ifac), ipnfbr(ifac+1)-1
  
    inod = nodfbr(ii)
    
    sf1_SW =     1/(sqrt(3.d0)*delta0)*(0.5*(L+sqrt(3.d0)*delta0) + xyzno0(1,inod)&
     - sqrt(3.d0)*(xyzno0(2,inod) + (D + delta0/2.d0)))
    sf3_SW = 1 - 2/(sqrt(3.d0)*delta0)*(0.5*(L+sqrt(3.d0)*delta0) + xyzno0(1,inod))
    sf2_SW =  1 - (sf1_SW + sf3_SW)
    
    if (impale(inod).eq.0) then
      
      disale(1, inod) = depl_S(1) * sf1_SW + depl_O(1) * sf2_SW + depl_SW(1) * sf3_SW
      disale(2, inod) = depl_S(2) * sf1_SW + depl_O(2) * sf2_SW + depl_SW(2) * sf3_SW
      disale(3, inod) = 0.d0
      
      impale(inod) = 1
      
    endif
    
  enddo 
  
enddo
! **********************************************************************

! ******************************* PLATE ******************************** 
call getfbr('IA_fC_SW_SE_plate or IA_fC_SW_SE_plate_top or IAcomp_fC_inlet_right_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac  = lstelt(ilelt)
   
  do ii = ipnfbr(ifac), ipnfbr(ifac+1)-1
  
    inod = nodfbr(ii)
    
    if (impale(inod).eq.0) then
  
      y_SW_comp = -sin(theta_SW_comp)*(xyzno0(1,inod) - (-xO_NE_comp)) + cos(theta_SW_comp)*(xyzno0(2,inod) - (-yO_NE_comp))
      
      disale(1, inod) = (depl_S(1) - depl_SW(1))/delta0 * y_SW_comp + (depl_S(1) + depl_SW(1))/2.d0
      disale(2, inod) = (depl_S(2) - depl_SW(2))/delta0 * y_SW_comp + (depl_S(2) + depl_SW(2))/2.d0
      disale(3, inod) = 0.d0
      
      impale(inod) = 1
      
    endif
    
  enddo 
  
enddo
! **********************************************************************

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! ################################################################################################################
! ################################################################################################################
!#endif

! Deallocate the temporary array
deallocate(lstelt)
! Deallocate the temporary array
deallocate(lstcel)

return
end subroutine usalcl
