!-------------------------------------------------------------------------------

!                      Code_Saturne version 5.0.8
!                      --------------------------
! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2018 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------

!===============================================================================
! Purpose:
! --------

!> \file cs_user_fluid_structure_interaction.f90
!>
!> \brief User subroutines dedicated to Fluid-Structure interaction modeling
!>
!>   \par Management of internal Fluid-Structure coupled calculations:
!>        a simplified solid model is used
!>        (linear "mass, friction and spring" modeling).
!>        Here are 2 differents subroutines that need to be filled :
!>
!>     - \ref usstr1 : Called at the beginning of the calculation. It enables
!>                     one to define internal structures and corresponding
!>                     initial conditions (initial displacement and velocity).
!>
!>     - \ref usstr2 : Called at each time step of the calculation. Here one
!>                     structural parameters
!>                     (considered to be potentially time dependent),
!>                     i.e. Mass, Friction, Stiffness and Fluid Stresses.
!>
!>   \par Fluid-Structure coupling with Code_Aster:
!>        the user subroutine \ref usaste has to be used.
!>
!>   \par Examples of data settings for fluid-structure interaction (FSI):
!>        Several examples are available
!>        \ref cs_user_fluid_structure_interaction "here".
!
!-------------------------------------------------------------------------------

!===============================================================================
! Function :
! ----------

!> \brief Definition of internal structures and corresponding initial conditions
!>       (initial displacement and velocity ).
!
!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]    idfstr         boundary faces -> structure definition
!> \param[in]    aexxst,bexxst  prediction coefficients of structural data
!> \param[in]    cfopre         prediction coefficients of fluid forces
!> \param[in]    xstr0          initial displacement of internal structures
!> \param[in]    vstr0          initial velocity of internal structures
!> \param[in]    xstreq         displacement of initial mesh compared to
!>                              structures position at equilibrium
!______________________________________________________________________________!

subroutine usstr1 &
 ( idfstr ,                                                       &
   aexxst , bexxst , cfopre ,                                     &
   xstr0  , vstr0  , xstreq )

!===============================================================================

!===============================================================================
! Module files
!===============================================================================

use paramx
use cstnum
use optcal
use entsor
use albase
use parall
use period
use mesh

!===============================================================================

implicit none

! Arguments

integer          idfstr(nfabor)

double precision aexxst, bexxst, cfopre
double precision xstr0(3,nstrmx), xstreq(3,nstrmx)
double precision vstr0(3,nstrmx)

! Local variables

double precision hn_FS , heq
double precision L_FS , D_FS
double precision theta_str
integer istr , ifac , ilelt , nlelt

integer, allocatable, dimension(:) :: lstelt

!===============================================================================

! Additional geometrical parameter (hexagon edge)
hn_FS  = 3.0d-3
heq = hn_FS

L_FS = 0.07332d0
D_FS = (sqrt(3.d0)/2.d0)*L_FS

!===============================================================================
! 1.  Initialization
!===============================================================================

! Allocate a temporary array for boundary faces selection
allocate(lstelt(nfabor))

!===============================================================================
! 2.  Definition of internal structures and initial displacement
!===============================================================================


! %%%%%%%%%%%%%%%%%%%%%%%%%% 2.0 - INLETS %%%%%%%%%%%%%%%%%%%%%%%%%%


call getfbr('IAcomp_fC_inlet_right_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)

  idfstr(ifac) = 0
  
enddo

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

#if 0
! %%%%%%%%%%%%%%%%%%%%% 2.1 - TOP/BOTTOM FACES %%%%%%%%%%%%%%%%%%%%%

call getfbr('IA_fC_SE_triangle or IA_fC_E_triangle or IA_fC_NE_triangle&
& or IA_fC_NW_triangle or IA_fC_W_triangle or IA_fC_SW_triangle or IA_fC_S_O_plate&
& or IA_fC_SE_O_plate or IA_fC_NE_O_plate or IA_fC_N_O_plate&
& or IA_fC_NW_O_plate or IA_fC_SW_O_plate&
& or IA_fC_SE_SE_plate or IA_fC_E_SE_plate or IA_fC_NE_SE_plate or IA_fC_NW_SE_plate&
& or IA_fC_W_SE_plate or IA_fC_SW_SE_plate', nlelt, lstelt)

do ilelt = 1, nlelt
  
  ifac  = lstelt(ilelt)
 
  idfstr(ifac) = 0
  
enddo 

call getfbr('IA_fC_SE_triangle_top or IA_fC_E_triangle_top or IA_fC_NE_triangle_top&
& or IA_fC_NW_triangle_top or IA_fC_W_triangle_top or IA_fC_SW_triangle_top&
& or IA_fC_S_O_plate_top or IA_fC_SE_O_plate_top& 
& or IA_fC_NE_O_plate_top or IA_fC_N_O_plate_top&
& or IA_fC_NW_O_plate_top or IA_fC_SW_O_plate_top or IA_fC_SE_SE_plate_top&
& or IA_fC_E_SE_plate_top or IA_fC_NE_SE_plate_top or IA_fC_NW_SE_plate_top&
& or IA_fC_W_SE_plate_top or IA_fC_SW_SE_plate_top', nlelt, lstelt)

do ilelt = 1, nlelt
  
  ifac  = lstelt(ilelt)
 
  idfstr(ifac) = 0
  
enddo 

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#endif

! %%%%%%%%%%%%%%%%%%%%%%%%%% 2.2 - CENTRAL HEXAGONE %%%%%%%%%%%%%%%%%%%%%%%%%%
 
call getfbr('IA_fC_S_top_extruded or IA_fC_SE_top_extruded or IA_fC_NE_top_extruded&
& or IA_fC_N_top_extruded or IA_fC_NW_top_extruded or IA_fC_SW_top_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)

  idfstr(ifac) = 1
  
enddo

istr = 1
   
xstreq(1,istr) = 0.d0
xstreq(2,istr) = 0.d0
xstreq(3,istr) = 0.d0 

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! %%%%%%%%%%%%%%%%%%%%%%%%% 2.3 - SOUTHERN HEXAGONE %%%%%%%%%%%%%%%%%%%%%%%%%%

call getfbr('IA_fC_hp_SW_top_extruded or IA_fC_S_bottom_extruded&
& or IA_fC_hp_SE_bottom_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)

  idfstr(ifac) = 2

enddo

istr = 2
theta_str = -pi/2.d0 + pi/3.d0*(istr-2)
   
xstreq(1,istr) = (heq/2.d0)*cos(theta_str)
xstreq(2,istr) = (heq/2.d0)*sin(theta_str)
xstreq(3,istr) = 0.d0

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! %%%%%%%%%%%%%%%%%%%%%%% 2.4 - SOUTHEASTERN HEXAGONE %%%%%%%%%%%%%%%%%%%%%%%%

call getfbr('IA_fC_hp_SE_top_extruded or IA_fC_SE_bottom_extruded&
& or IA_fC_hp_E_bottom_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)

  idfstr(ifac) = 3

enddo

istr = 3
theta_str = -pi/2.d0 + pi/3.d0*(istr-2)
   
xstreq(1,istr) = (heq/2.d0)*cos(theta_str)
xstreq(2,istr) = (heq/2.d0)*sin(theta_str)
xstreq(3,istr) = 0.d0

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! %%%%%%%%%%%%%%%%%%%%%%% 2.5 - NORTHEASTERN HEXAGONE %%%%%%%%%%%%%%%%%%%%%%%%

call getfbr('IA_fC_hp_E_top_extruded or IA_fC_NE_bottom_extruded&
& or IA_fC_hp_NE_bottom_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)

  idfstr(ifac) = 4

enddo

istr = 4
theta_str = -pi/2.d0 + pi/3.d0*(istr-2)
   
xstreq(1,istr) = (heq/2.d0)*cos(theta_str)
xstreq(2,istr) = (heq/2.d0)*sin(theta_str)
xstreq(3,istr) = 0.d0

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! %%%%%%%%%%%%%%%%%%%%%%%%% 2.6 - NORTHERN HEXAGONE %%%%%%%%%%%%%%%%%%%%%%%%%%

call getfbr('IA_fC_hp_NE_top_extruded or IA_fC_N_bottom_extruded&
& or IA_fC_hp_NW_bottom_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)

  idfstr(ifac) = 5

enddo

istr = 5
theta_str = -pi/2.d0 + pi/3.d0*(istr-2)
   
xstreq(1,istr) = (heq/2.d0)*cos(theta_str)
xstreq(2,istr) = (heq/2.d0)*sin(theta_str)
xstreq(3,istr) = 0.d0

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! %%%%%%%%%%%%%%%%%%%%%%% 2.7 - NORTHWESTERN HEXAGONE %%%%%%%%%%%%%%%%%%%%%%%%

call getfbr('IA_fC_hp_NW_top_extruded or IA_fC_NW_bottom_extruded&
& or IA_fC_hp_W_bottom_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)

  idfstr(ifac) = 6

enddo

istr = 6
theta_str = -pi/2.d0 + pi/3.d0*(istr-2)
   
xstreq(1,istr) = (heq/2.d0)*cos(theta_str)
xstreq(2,istr) = (heq/2.d0)*sin(theta_str)
xstreq(3,istr) = 0.d0

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


! %%%%%%%%%%%%%%%%%%%%%%% 2.8 - SOUTHWESTERN HEXAGONE %%%%%%%%%%%%%%%%%%%%%%%%

call getfbr('IA_fC_hp_W_top_extruded or IA_fC_SW_bottom_extruded&
& or IA_fC_hp_SW_bottom_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)

  idfstr(ifac) = 7

enddo

istr = 7
theta_str = -pi/2.d0 + pi/3.d0*(istr-2)
   
xstreq(1,istr) = (heq/2.d0)*cos(theta_str)
xstreq(2,istr) = (heq/2.d0)*sin(theta_str)
xstreq(3,istr) = 0.d0

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! Deallocate the temporary array
deallocate(lstelt)

return
end subroutine usstr1


!===============================================================================
! Purpose :
! ---------

!> \brief Definition of structural parameters in case of Fluid Structure
!>        internal coupling : Mass, Friction, Stiffness anf Fluid Stresses.
!
!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]    nbstru         nombre de structures definies
!> \param[in]    idfstr         definition des structures
!> \param[in]    dtcel          time step (per cell
!> \param[in]    xmstru         matrix of structural mass
!> \param[out]   xcstru         matrix of structural friction
!> \param[out]   xkstru         matrix of structural stiffness
!> \param[in]    xstreq         displacement of initial mesh compared to
!> \param[in]    xstr           structural displacement
!> \param[in]    vstr           structural velocity
!> \param[in]    forstr         forces acting on structures (take forces)
!> \param[out]   dtstr          structural time step
!______________________________________________________________________________!

subroutine usstr2 &
 ( nbstru ,                                                       &
   idfstr ,                                                       &
   dtcel  ,                                                       &
   xmstru , xcstru , xkstru , xstreq , xstr   , vstr   , forstr , &
   dtstr  )

!===============================================================================

!===============================================================================
! Module files
!===============================================================================

use paramx
use cstnum
use optcal
use albase
use parall
use period
use mesh

!===============================================================================

implicit none

! Arguments

integer          nbstru

integer          idfstr(nfabor)

double precision dtcel(ncelet)
double precision xmstru(3,3,nstrmx)
double precision xcstru(3,3,nstrmx)
double precision xkstru(3,3,nstrmx)
double precision xstreq(3,nstrmx)
double precision xstr(3,nstrmx)
double precision vstr(3,nstrmx)
double precision forstr(3,nstrmx)
double precision dtstr(nstrmx)

! Local variables

integer          ii, jj, istr
double precision hexa_m_lin , hexa_c_lin , hexa_k_lin

!===============================================================================

! ************************************
! Definition of structure's parameters
! ************************************

hexa_m_lin = 85.22d0
hexa_c_lin = 0.d0
hexa_k_lin = 2.526d4

! hexa_m_lin = 1.d1
! hexa_c_lin = 0.d0
! hexa_k_lin = 1.d2

!===============================================================================
! Definition of structural parameters: mass, friction, stiffness and stresses.
!===============================================================================

! --- Matrices xmstru, xcstru and xkstru are initialized to the value of 0.
do istr = 1, nbstru

  do ii = 1, 3
    do jj = 1, 3
      xmstru(ii,jj,istr) = 0.d0
      xcstru(ii,jj,istr) = 0.d0
      xkstru(ii,jj,istr) = 0.d0
    enddo
  enddo

enddo

! Isotropic modeling
do istr = 1, nbstru

  do ii = 1, 3
    xmstru(ii,ii,istr) = hexa_m_lin
    xcstru(ii,ii,istr) = hexa_c_lin
    xkstru(ii,ii,istr) = hexa_k_lin
  enddo
  
enddo

return

end subroutine usstr2

!===============================================================================
