!-------------------------------------------------------------------------------

!                      Code_Saturne version 5.0.8
!                      --------------------------
! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2018 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------

!===============================================================================
! Purpose:
! -------

!> \file cs_user_initialization.f90
!>
!> \brief Initialize variables
!>
!> See \subpage cs_user_initialization for examples.
!>
!> This subroutine is called at beginning of the computation
!> (restart or not) before the loop time step.
!>
!> This subroutine enables to initialize or modify (for restart)
!> unkown variables and time step values.
!>
!> Modification of the behaviour law of physical quantities (rom, viscl,
!> viscls, cp) is not done here. It is the purpose of the user subroutine
!> \ref usphyv (in cs_user_physical_properties.f90)
!>
!> \c rom and \c viscl values are equal to \c ro0 and \c viscl0 or initialize
!> by reading the restart file.
!> Variables diffusivity and specific heat (when they are defined) have no value
!> except if they are read from a restart file.
!>
!> \par cs_user_initialization_cell_id Cells identification
!>
!> Cells may be identified using the \ref getcel subroutine.
!> The syntax of this subroutine is described in the
!> \ref cs_user_boundary_conditions subroutine,
!> but a more thorough description can be found in the user guide.
!>
!> Field cell values arrays can be retrieved using the appropriate access
!> functions as described \ref field "here".
!>
!> Example of field ids:
!> - Density:                        \c irom
!> - Dynamic molecular viscosity:    \c iviscl
!> - Turbulent viscosity:            \c ivisct
!> - Specific heat:                  \c icp
!> - Diffusivity(lambda):            \c field_get_key_int(ivarfl(isca(iscal)),
!>                                      kivisl, ...)
!>
!
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]     nvar          total number of variables
!> \param[in]     nscal         total number of scalars
!> \param[in]     dt            time step (per cell)
!_______________________________________________________________________________

subroutine cs_user_f_initialization &
 ( nvar   , nscal  ,                                              &
   dt     )

!===============================================================================

!===============================================================================
! Module files
!===============================================================================

use paramx
use pointe
use numvar
use optcal
use cstphy
use cstnum
use entsor
use parall
use period
use ppppar
use ppthch
use coincl
use cpincl
use ppincl
use atincl
use ctincl
use ppcpfu
use cs_coal_incl
use cs_fuel_incl
use mesh
use field
use turbomachinery

!===============================================================================

implicit none

! Arguments

integer          nvar   , nscal

double precision dt(ncelet)

! Local variables

! INSERT_VARIABLE_DEFINITIONS_HERE

integer, allocatable, dimension(:) :: lstelt

!< [loc_var_dec]
integer          ifac, iel, ii
integer          inod
integer          ilelt, nlelt
integer          ilcel, nlcel

double precision, dimension(:), pointer :: cvar_pr
double precision, dimension(:,:), pointer :: cvar_vel
double precision, dimension(:,:), pointer :: cvar_mvisc
double precision, dimension(:,:), pointer :: cvar_meshvel
! Declaration of the parameters of the forcing velocity term
!-----------------------------------------------------------
double precision f_MV, w_MV, hn_MV, a_MV
double precision deltaa, delta0, dhdt
!-----------------------------------------------------------

! Declaration of the length of the hexagon edges
!-----------------------------------------------
double precision L , D , semiP0
!-----------------------------------------------

! Declaration of the parameters for azimuts
!------------------------------------------
double precision theta_NE
double precision theta_IA_SE , theta_IA_NE , theta_IA_NW , theta_IA_SW
double precision theta_SE_comp , theta_NE_comp , theta_NW_comp , theta_SW_comp
!------------------------------------------

! Declaration of the parameters for displacement
!-----------------------------------------------------------
double precision depl , deplNE_x , deplNE_y
!-----------------------------------------------------------

! Nodes' coordinates with respect to the reference axis system
!-------------------------------------------------------------
double precision xO_NE , yO_NE
double precision xO_NW , yO_NW
double precision xO_SW , yO_SW
double precision xO_SE , yO_SE
!-------------------------------------------------------------

! Nodes' coordinates with respect to the reference axis system
!-------------------------------------------------------------
double precision xO_NE_comp , yO_NE_comp
!-------------------------------------------------------------

! Nodes' coordinates with respect to the local axis system
!---------------------------------------------------------
double precision y_SE , y_NE , y_NW , y_SW
double precision y_SE_comp , y_NE_comp , y_NW_comp , y_SW_comp
!---------------------------------------------------------

! Shape functions for the triangular zone
!----------------------------------------
double precision sf1_SE , sf2_SE , sf3_SE
double precision sf1_E  , sf2_E  , sf3_E
double precision sf1_NE , sf2_NE , sf3_NE
double precision sf1_NW , sf2_NW , sf3_NW
double precision sf1_W  , sf2_W  , sf3_W
double precision sf1_SW , sf2_SW , sf3_SW
!----------------------------------------

! Local parameters for radial plates
!-----------------------------------
double precision theta_comp , depl_comp_x , depl_comp_y
!-----------------------------------

! Local parameters for mesh viscosity
!------------------------------------
double precision viscmin , viscmax
double precision plate_thres , tri_thres
!-----------------------------------

!< [loc_var]
!===============================================================================

! Calculation of displacement at current time step

!< [calcul]

! Additional geometrical parameter (hexagon edge)
L = 0.07332d0
D = (sqrt(3.d0)/2.d0)*L

delta0    = hn_MV*(1+a_MV)

semiP0 = D + delta0/2.d0
! Orientation

! Localisation polaire
theta_NE      = pi/6

! Orientation des repères
theta_IA_SE   =   pi/3
theta_IA_NE   = 2*pi/3
theta_IA_NW   = 4*pi/3
theta_IA_SW   = 5*pi/3

theta_SE_comp =  -pi/3
theta_NE_comp =   pi/3
theta_NW_comp = 2*pi/3
theta_SW_comp = 4*pi/3

! Spatial coordinates of the inter-spaces center (radial)
xO_NE_comp = (1.5d0*L + sqrt(3.d0)/2.d0*delta0)*cos(theta_NE_comp) 
yO_NE_comp = (1.5d0*L + sqrt(3.d0)/2.d0*delta0)*sin(theta_NE_comp)  

! Spatial coordinates of the inter-spaces center (orthoradial)
xO_NE = semiP0*cos(theta_NE)
yO_NE = semiP0*sin(theta_NE)  

!< [calcul]

! Grid velocity with respect to the local coordinates on radial plates (independent of the location the plate)
theta_comp = pi/6

!< [loc_var_dec]

!===============================================================================


!===============================================================================
! Initialization
!===============================================================================

allocate(lstelt(ncel)) ! temporary array for cells selection

! INSERT_MAIN_CODE_HERE

!--------
! Formats
!--------

! Pressure initialization

!#if 0
call field_get_val_s(ivarfl(ipr), cvar_pr)

do iel = 1, ncel
  cvar_pr(iel)  = 1.0d5
enddo
!#endif

! Velocity initialization (by component)

call field_get_val_v(ivarfl(iu), cvar_vel)
call field_get_val_v(ivarfl(iuma), cvar_meshvel)

do iel = 1, ncel
!  cvar_vel(1,ifac)  = 1.0d-3
  cvar_vel(1,ifac)  = 0.
  cvar_vel(2,ifac)  = 0.
  cvar_vel(3,ifac)  = 0.
   
  cvar_meshvel(1,ifac)  = 0.
  cvar_meshvel(2,ifac)  = 0.
  cvar_meshvel(3,ifac)  = 0.
enddo

#if 0
call field_get_val_v_by_name("mesh_viscosity",cvar_mvisc)

viscmax = 1.0d10
viscmin = 1.0d0

plate_thres = 0.9d0 ! 1.d0 
tri_thres   = 0.1d0 ! 0.d0 

! call field_get_val_v(ivarfl(ivisma), cvar_mvisc)

#if 0
call getcel('all[]', nlelt, lstelt)

do ilelt = 1, nlelt
  iel  = lstelt(ilelt)
  cvar_mvisc(1,iel)  = 1.0d10
  cvar_mvisc(2,iel)  = 1.0d10
  cvar_mvisc(3,iel)  = 1.0d10
enddo
#endif


!#if 0
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%% TRIANGLES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
call getcel('IA_fC_SE_triangle_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  iel  = lstelt(ilelt)
    
  sf1_SE = 1 - 2/(sqrt(3.d0)*delta0)*(0.5*(L+sqrt(3.d0)*delta0) - xyzcen(1,iel))
  sf3_SE =     1/(sqrt(3.d0)*delta0)*(0.5*(L+sqrt(3.d0)*delta0) - xyzcen(1,iel)&
   - sqrt(3.d0)*(xyzcen(2,iel) + (D + delta0/2.d0)))
  sf2_SE =  1 - (sf1_SE + sf3_SE)
  
#if 0
  cvar_mvisc(1,iel) = viscmax + (viscmin - viscmax)  * (sf1_SE * sf2_SE * sf3_SE) * (1/3.d0)**(-3.d0)
  cvar_mvisc(2,iel) = viscmax + (viscmin - viscmax)  * (sf1_SE * sf2_SE * sf3_SE) * (1/3.d0)**(-3.d0)
#endif  

  ! cvar_mvisc(1,iel)  = 1.0d10
  ! cvar_mvisc(2,iel)  = 1.0d10
  
!#if 0
  if ( abs((sf1_SE * sf2_SE * sf3_SE) * (1/3.d0)**(-3.d0)) .lt. tri_thres ) then
    cvar_mvisc(1,iel) = viscmax
    cvar_mvisc(2,iel) = viscmax
  endif
!#endif

enddo

call getcel('IA_fC_E_triangle_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  iel  = lstelt(ilelt)
    
  sf1_E = -1/(sqrt(3.d0)*delta0)*( (L- xyzcen(1,iel)) - sqrt(3.d0)*xyzcen(2,iel) )
  sf3_E = -1/(sqrt(3.d0)*delta0)*( (L- xyzcen(1,iel)) + sqrt(3.d0)*xyzcen(2,iel) )
  sf2_E =  1 - (sf1_E + sf3_E)
  
#if 0
  cvar_mvisc(1,iel) = viscmax + (viscmin - viscmax)  * (sf1_E * sf2_E * sf3_E) * (1/3.d0)**(-3.d0)
  cvar_mvisc(2,iel) = viscmax + (viscmin - viscmax)  * (sf1_E * sf2_E * sf3_E) * (1/3.d0)**(-3.d0)
#endif  

  ! cvar_mvisc(1,iel)  = 1.0d10
  ! cvar_mvisc(2,iel)  = 1.0d10
  
!#if 0
  if ( abs((sf1_E * sf2_E * sf3_E) * (1/3.d0)**(-3.d0)) .lt. tri_thres ) then
    cvar_mvisc(1,iel) = viscmax
    cvar_mvisc(2,iel) = viscmax
  endif
!#endif

enddo

call getcel('IA_fC_NE_triangle_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  iel  = lstelt(ilelt)
    
    sf1_NE =     1/(sqrt(3.d0)*delta0)*(0.5*(L+sqrt(3.d0)*delta0) - xyzcen(1,iel)&
     - sqrt(3.d0)*(-xyzcen(2,iel) + (D + delta0/2.d0)))
    sf3_NE = 1 - 2/(sqrt(3.d0)*delta0)*(0.5*(L+sqrt(3.d0)*delta0) - xyzcen(1,iel))
    sf2_NE =  1 - (sf1_NE + sf3_NE)
    
#if 0
  cvar_mvisc(1,iel) = viscmax + (viscmin - viscmax)  * (sf1_NE * sf2_NE * sf3_NE) * (1/3.d0)**(-3.d0)
  cvar_mvisc(2,iel) = viscmax + (viscmin - viscmax)  * (sf1_NE * sf2_NE * sf3_NE) * (1/3.d0)**(-3.d0)
#endif  

  ! cvar_mvisc(1,iel)  = 1.0d10
  ! cvar_mvisc(2,iel)  = 1.0d10
  
!#if 0
  if ( abs((sf1_NE * sf2_NE * sf3_NE) * (1/3.d0)**(-3.d0)) .lt. tri_thres ) then
    cvar_mvisc(1,iel) = viscmax
    cvar_mvisc(2,iel) = viscmax
  endif
!#endif
  
enddo

call getcel('IA_fC_NW_triangle_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  iel  = lstelt(ilelt)
    
  sf1_NW = 1 - 2/(sqrt(3.d0)*delta0)*(0.5*(L+sqrt(3.d0)*delta0) + xyzcen(1,iel))
  sf3_NW =     1/(sqrt(3.d0)*delta0)*(0.5*(L+sqrt(3.d0)*delta0) + xyzcen(1,iel)&
   - sqrt(3.d0)*(-xyzcen(2,iel) + (D + delta0/2.d0)))
  sf2_NW =  1 - (sf1_NW + sf3_NW)

#if 0
  cvar_mvisc(1,iel) = viscmax + (viscmin - viscmax)  * (sf1_NW * sf2_NW * sf3_NW) * (1/3.d0)**(-3.d0)
  cvar_mvisc(2,iel) = viscmax + (viscmin - viscmax)  * (sf1_NW * sf2_NW * sf3_NW) * (1/3.d0)**(-3.d0)
#endif  

  ! cvar_mvisc(1,iel)  = 1.0d10
  ! cvar_mvisc(2,iel)  = 1.0d10
  
!#if 0
  if ( abs((sf1_NW * sf2_NW * sf3_NW) * (1/3.d0)**(-3.d0)) .lt. tri_thres ) then
    cvar_mvisc(1,iel) = viscmax
    cvar_mvisc(2,iel) = viscmax
  endif
!#endif
     
enddo 

call getcel('IA_fC_W_triangle_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  iel  = lstelt(ilelt)
    
  sf1_W = -1/(sqrt(3.d0)*delta0)*( (L + xyzcen(1,iel)) + sqrt(3.d0)*xyzcen(2,iel) )
  sf3_W = -1/(sqrt(3.d0)*delta0)*( (L + xyzcen(1,iel)) - sqrt(3.d0)*xyzcen(2,iel) )
  sf2_W =  1 - (sf1_W + sf3_W)  
  
#if 0
  cvar_mvisc(1,iel) = viscmax + (viscmin - viscmax)  * (sf1_W * sf2_W * sf3_W) * (1/3.d0)**(-3.d0)
  cvar_mvisc(2,iel) = viscmax + (viscmin - viscmax)  * (sf1_W * sf2_W * sf3_W) * (1/3.d0)**(-3.d0)
#endif  

  ! cvar_mvisc(1,iel)  = 1.0d10
  ! cvar_mvisc(2,iel)  = 1.0d10
  
!#if 0
  if ( abs((sf1_W * sf2_W * sf3_W) * (1/3.d0)**(-3.d0)) .lt. tri_thres ) then
    cvar_mvisc(1,iel) = viscmax
    cvar_mvisc(2,iel) = viscmax
  endif
!#endif
    
enddo

call getcel('IA_fC_SW_triangle_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  iel  = lstelt(ilelt)
    
  sf1_SW =     1/(sqrt(3.d0)*delta0)*(0.5*(L+sqrt(3.d0)*delta0) + xyzcen(1,iel)&
   - sqrt(3.d0)*(xyzcen(2,iel) + (D + delta0/2.d0)))
  sf3_SW = 1 - 2/(sqrt(3.d0)*delta0)*(0.5*(L+sqrt(3.d0)*delta0) + xyzcen(1,iel))
  sf2_SW =  1 - (sf1_SW + sf3_SW)  

#if 0
  cvar_mvisc(1,iel) = viscmax + (viscmin - viscmax)  * (sf1_SW * sf2_SW * sf3_SW) * (1/3.d0)**(-3.d0)
  cvar_mvisc(2,iel) = viscmax + (viscmin - viscmax)  * (sf1_SW * sf2_SW * sf3_SW) * (1/3.d0)**(-3.d0)
#endif  

  ! cvar_mvisc(1,iel)  = 1.0d10
  ! cvar_mvisc(2,iel)  = 1.0d10
  
!#if 0
  if ( abs((sf1_SW * sf2_SW * sf3_SW) * (1/3.d0)**(-3.d0)) .lt. tri_thres ) then
    cvar_mvisc(1,iel) = viscmax
    cvar_mvisc(2,iel) = viscmax
  endif
!#endif
    
enddo  
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!#endif

!#if 0
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%% PLATES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

call getcel('IA_fC_S_O_plate_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  iel = lstelt(ilelt)
    
  ! cvar_mvisc(2,iel) = viscmax + (viscmin - viscmax)*abs((xyzcen(2,iel)+semiP0))/delta0/2.d0
  
  if ( abs((abs(xyzcen(2,iel))-semiP0))/delta0/2.d0 .gt. plate_thres ) then
    cvar_mvisc(1,iel) = viscmax
    cvar_mvisc(2,iel) = viscmax
  endif
  
enddo

call getcel('IA_fC_SE_SE_plate_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  iel = lstelt(ilelt)
  
  y_SE_comp = -sin(theta_SE_comp)*(xyzcen(1,iel) - xO_NE_comp) + cos(theta_SE_comp)*(xyzcen(2,iel) - (-yO_NE_comp))
  
  if ( abs(y_SE_comp)/delta0/2.d0 .gt. plate_thres ) then
    cvar_mvisc(1,iel) = viscmax
    cvar_mvisc(2,iel) = viscmax
  endif
  
enddo

call getcel('IA_fC_SE_O_plate_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  iel = lstelt(ilelt)
  
  y_SE = -sin(theta_IA_SE)*(xyzcen(1,iel) - xO_NE) + cos(theta_IA_SE)*(xyzcen(2,iel) - (-yO_NE))
  
   if ( abs(y_SE)/delta0/2.d0 .gt. plate_thres ) then
    cvar_mvisc(1,iel) = viscmax
    cvar_mvisc(2,iel) = viscmax
  endif
  
enddo

call getcel('IA_fC_E_SE_plate_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  iel = lstelt(ilelt)

  if ( abs(xyzcen(2,iel))/delta0/2.d0 .gt. plate_thres ) then
    cvar_mvisc(1,iel) = viscmax
    cvar_mvisc(2,iel) = viscmax
  endif
  
enddo

call getcel('IA_fC_NE_O_plate_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  iel = lstelt(ilelt)
  
  y_NE = -sin(theta_IA_NE)*(xyzcen(1,iel) - xO_NE) + cos(theta_IA_NE)*(xyzcen(2,iel) - yO_NE)
  
  if ( abs(y_NE)/delta0/2.d0 .gt. plate_thres ) then
    cvar_mvisc(1,iel) = viscmax
    cvar_mvisc(2,iel) = viscmax
  endif
  
enddo

call getcel('IA_fC_NE_SE_plate_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  iel = lstelt(ilelt)
  
  y_NE_comp = -sin(theta_NE_comp)*(xyzcen(1,iel) - xO_NE_comp) + cos(theta_NE_comp)*(xyzcen(2,iel) - yO_NE_comp)
  
  if ( abs(y_NE_comp)/delta0/2.d0 .gt. plate_thres ) then
    cvar_mvisc(1,iel) = viscmax
    cvar_mvisc(2,iel) = viscmax
  endif
  
enddo

call getcel('IA_fC_N_O_plate_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  iel = lstelt(ilelt)
  
  if ( abs((abs(xyzcen(2,iel))-semiP0))/delta0/2.d0 .gt. plate_thres ) then
    cvar_mvisc(1,iel) = viscmax
    cvar_mvisc(2,iel) = viscmax
  endif
  
enddo

call getcel('IA_fC_NW_SE_plate_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  iel = lstelt(ilelt)
  
  y_NW_comp = -sin(theta_NW_comp)*(xyzcen(1,iel) - (-xO_NE_comp)) + cos(theta_NW_comp)*(xyzcen(2,iel) - yO_NE_comp)
  
  if ( abs(y_NW_comp)/delta0/2.d0 .gt. plate_thres ) then
    cvar_mvisc(1,iel) = viscmax
    cvar_mvisc(2,iel) = viscmax
  endif
  
enddo

call getcel('IA_fC_NW_O_plate_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  iel = lstelt(ilelt)
  
  y_NW = -sin(theta_IA_NW)*(xyzcen(1,iel) - (-xO_NE)) + cos(theta_IA_NW)*(xyzcen(2,iel) - yO_NE)
  
  if ( abs(y_NW)/delta0/2.d0 .gt. plate_thres ) then
    cvar_mvisc(1,iel) = viscmax
    cvar_mvisc(2,iel) = viscmax
  endif
  
enddo

call getcel('IA_fC_W_SE_plate_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  iel = lstelt(ilelt)

  if ( abs(xyzcen(2,iel))/delta0/2.d0 .gt. plate_thres ) then
    cvar_mvisc(1,iel) = viscmax
    cvar_mvisc(2,iel) = viscmax
  endif
  
enddo

call getcel('IA_fC_SW_O_plate_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  iel = lstelt(ilelt)
  
   y_SW = -sin(theta_IA_SW)*(xyzcen(1,iel) - (-xO_NE)) + cos(theta_IA_SW)*(xyzcen(2,iel) - (-yO_NE))
  
  if ( abs(y_SW)/delta0/2.d0 .gt. plate_thres ) then
    cvar_mvisc(1,iel) = viscmax
    cvar_mvisc(2,iel) = viscmax
  endif
  
enddo

call getcel('IA_fC_SW_SE_plate_extruded', nlelt, lstelt)

do ilelt = 1, nlelt

  iel = lstelt(ilelt)
  
   y_SW_comp = -sin(theta_SW_comp)*(xyzcen(1,iel) - (-xO_NE_comp)) + cos(theta_SW_comp)*(xyzcen(2,iel) - (-yO_NE_comp))
  
  if ( abs(y_SW_comp)/delta0/2.d0 .gt. plate_thres ) then
    cvar_mvisc(1,iel) = viscmax
    cvar_mvisc(2,iel) = viscmax
  endif
  
enddo

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!#endif
#endif

!----
! End
!----

deallocate(lstelt) ! temporary array for cells selection

return
end subroutine cs_user_f_initialization
