!-------------------------------------------------------------------------------

!                      Code_Saturne version 5.0.8
!                      --------------------------
! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2018 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------

!===============================================================================

!> \file cs_user_parameters.f90
!>
!> \brief User subroutines for input of calculation parameters (Fortran modules).
!>        These subroutines are called in all cases.
!>
!>  See \subpage f_parameters for examples.
!>
!>   If the Code_Saturne GUI is used, this file is not required (but may be
!>   used to override parameters entered through the GUI, and to set
!>   parameters not accessible through the GUI).
!>
!>   Several routines are present in the file, each destined to defined
!>   specific parameters.
!>
!>   To modify the default value of parameters which do not appear in the
!>   examples provided, code should be placed as follows:
!>   - usipsu   for numerical and physical options
!>   - usipes   for input-output related options
!>
!>   As a convention, "specific physics" defers to the following modules only:
!>   pulverized coal, gas combustion, electric arcs.
!>
!>   In addition, specific routines are provided for the definition of some
!>   "specific physics" options.
!>   These routines are described at the end of this file and will be activated
!>   when the corresponding option is selected in the usppmo routine.
!-------------------------------------------------------------------------------

!===============================================================================

!> \brief User subroutine for selection of specific physics module

!> Define the use of a specific physics amongst the following:
!>   - combustion with gas / coal / heavy fuel oil
!>   - compressible flows
!>   - electric arcs
!>   - atmospheric modelling
!>   - radiative transfer
!>   - cooling towers modelling
!>
!>    Only one specific physics module can be activated at once.

!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]     ixmlpu        indicates if the XML file from the GUI is used
!>                              (1 : yes, 0 : no)
!______________________________________________________________________________!

subroutine usppmo &
 ( ixmlpu )

!===============================================================================
! Module files
!===============================================================================

use paramx
use entsor
use cstphy
use ppppar
use ppthch
use ppincl
use ppcpfu
use coincl
use radiat
use cs_c_bindings

!===============================================================================

implicit none

! Arguments

integer ixmlpu

return
end subroutine usppmo


!===============================================================================

!> \brief User subroutine for input of model selection parameters.

!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]      ixmlpu       indicates if the XML file from the GUI is used
!>                              used (1: yes, 0: no
!> \param[in, out] iturb        turbulence model
!> \param[in, out] itherm       thermal model
!> \param[in, out] iale         ale module
!> \param[in, out] ivofmt       vof method
!> \param[in, out] icavit       cavitation model
!______________________________________________________________________________!

subroutine usipph &
 ( ixmlpu, iturb , itherm, iale , ivofmt, icavit )

!===============================================================================
! Module files
!===============================================================================

use entsor, only: nfecra ! No other module should appear here
use optcal, only: irijco ! No other module should appear here

!===============================================================================

implicit none

! Arguments

integer ixmlpu
integer iturb, itherm, iale, ivofmt, icavit

! Local variables

!===============================================================================

!>    In this subroutine, only the parameters which already appear may
!>    be set, to the exclusion of any other.
!>
!>    If we are not using the Code_Saturne GUI:
!>    All the parameters which appear in this subroutine must be set.
!>
!>    If we are using the Code_Saturne GUI:
!>    parameters protected by a test of the form:
!>
!>      if (ixmlpu.eq.0) then
!>         ...
!>      endif
!>
!>    should already have been defined using the GUI, so only
!>    experts should consider removing the test and adapting them here.

!===============================================================================

!< [usipph]

! ------------------------------------------------------------------------
! Case:     Moving plates (2D)
! Purpose:  Setup of the corresponding input physcial parameters and options
! Remark:   For the explanation of the different input arguments of the subroutine
!           usipph, see ./EXAMPLES/cs_user_parameters.f90 
! ------------------------------------------------------------------------

! Workspace using input files (no GUI)
ixmlpu =  0

! No turbulence modelling is used -> for a relative medium-high Reynolds number,
! the refined mesh which is used is eventually expected to simulate a certain part
! of the Kolmogorov spectrum (hypothesis)
iturb =   0

! No effect of temperature is taken into account
itherm =  0

! Activation of ALE (Arbitrary Lagrangian Eulerian) method
iale =    1 ! Mouvement inter-plaques
! iale =    0 ! Cas test Poiseuille

! No VOF method is used
ivofmt =  -1

! Cavitation module: for now, no cavitation model is used
icavit = -1

!< [usipph]

!----
! Formats
!----

return
end subroutine usipph


!===============================================================================

!> \brief User subroutine for the input of additional user parameters.
!
!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]     nmodpp         number of active specific physics models
!______________________________________________________________________________!

subroutine usipsu &
 ( nmodpp )

!===============================================================================
! Module files
!===============================================================================

use paramx
use cstnum
use dimens
use numvar
use optcal
use cstphy
use entsor
use parall
use period
use ihmpre
use albase
use ppppar
use ppthch
use ppincl
use coincl
use cpincl
use field
use cavitation
use post
use rotation
use cs_c_bindings

!===============================================================================

implicit none

! Arguments

integer nmodpp

! 06/08/18 - Local variables for fields post-processing

integer ii
integer f_id, idim1, ifllog

type(var_cal_opt) :: vcopt

!===============================================================================

!>     This subroutine allows setting parameters
!>     which do not already appear in the other subroutines of this file.
!>
!>     It is possible to add or remove parameters.
!>     The number of physical properties and variables is known here.

!===============================================================================

!< [usipsu]

! Physical constants (cstphy)
! =========================== 

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! Reference fluid properties
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! ro0:    constant fluid density of water at T=20°C and P=1bar
! not necessary to be set considering a incompressible flow but helpful
! for the post-processing of the simulation results
! viscl0: constant molecular viscosity of water at T=20°C (m².s⁻¹)
! p0:     initial pressure value in the domain (Pa) -> Set to 1bar
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ro0 =     1.0d3
viscl0 =  1.0d-3
p0 =      1.0d5

! Calculation options (optcal)
! ============================

! In case of restart, read auxiliary restart file ileaux (= 1) or not (0)
ileaux = 0

! %%%%%%%%%%%%%
! Time stepping
! %%%%%%%%%%%%%

! Time scheme
! ischtp = 2

! Time step setup: uniform and constant such as in Cast3M-and FreeFem++'s setup,
idtvar = 0

! Reference time step: based on Cast3M-and FreeFem++'s ones
dtref = 1.d-5

! Simulation duration corresponding to twice the period value of the plates movement
! f = 20Hz => T = 0.05s
! ntmabs = 2/(f*dtref)
ntmabs = 1.0d4
! ntmabs = 2.0d4
! ntmabs = 1.0d3 ! Time focusing to get the minimum pressure value
! ntmabs = 10 ! Test
 
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! Additional options related to the pressure field
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! Hydrostatic pressure not taken into account

#if 0

iphydr

improve static pressure algorithm Take into account the balance or imbalance between the pressure gradient and source terms (as gravity and head losses)

    1: impose the equilibrium of the static part of the pressure with any external force, even head losses
    0: no treatment (default)
    2: hydrostatic pressure computation with a apriori momentum equation to obtain a hydrostatic pressure taking into account the imbalance between the pressure gradient and the gravity source term.

    When the density effects are important, the choice of iphydr = 1 allows to improve the interpolation of the pressure and correct the non-physical velocities which may appear in highly stratified areas or near horizontal walls (thus avoiding the use of extrag if the non-physical velocities are due only to gravity effects).
    The improved algorithm also allows eradicating the velocity oscillations which tend to appear at the frontiers of areas with high head losses.
    In the case of a stratified flow, the calculation cost is higher when the improved algorithm is used (about 30% depending on the case) because the hydrostatic pressure must be recalculated at the outlet boundary conditions: see icalhy.
    On meshes of insufficient quality, in order to improve the convergence, it may be useful to increase the number of iterations for the reconstruction of the pressure right-hand side, i.e. nswrsm.
    If head losses are present just along an outlet boundary, it is necessary to specify icalhy = 0 in order to deactivate the recalculation of the hydrostatic pressure at the boundary, which may otherwise cause instabilities. Please refer to the handling of the hydrostatic pressure section of the theory guide for more informations.

#endif

! iphydr = 0

#if 0

icalhy

compute the hydrostatic pressure in order to compute the Dirichlet conditions on the pressure at outlets

    1: calculation of the hydrostatic pressure at the outlet boundary
    0: no calculation of the hydrostatic pressure at the outlet boundary (default) This option is automatically specified depending on the choice of iphydr and the value of gravity (icalhy = 1 if iphydr = 1 and gravity is different from 0; otherwise icalhy = 0). The activation of this option generates an additional calculation cost (about 30% depending on the case).
    If head losses are present just along an outlet boundary, it is necessary to specify icalhy = 0 in order to deactivate the recalculation of the hydrostatic pressure at the boundary, which may otherwise cause instabilities
    
#endif

icalhy = 1

! igprij = 1
! Thermodynamical pressure (pther): by default initialized with p0 = p_atmos
! ipthrm = 0

! No porosity taken into account (Standard)
iporos = 0

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%
! VELOCITY / PRESSURE COUPLING
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%
! 0 : classical algorithm
! 1 : transient coupling
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%
ipucou = 0

! Velocity field

! Convective scheme: decentered upwind scheme
call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt)
! vcopt%iconv   = 1
! vcopt%ischcv  = 1
vcopt%blencv  = 0 ! -> Fully upwind scheme if blencv = 0
vcopt%thetav  = 1 ! -> Implicit first-order time scheme when theta = 1
! vcopt%ibdtso  = 2
call field_set_key_struct_var_cal_opt(ivarfl(iu), vcopt)

! Pressure field 

#if 0
call field_get_key_struct_var_cal_opt(ivarfl(isca(1)), vcopt)
vcopt%iconv   = 1
! vcopt%ischcv  = 1
! vcopt%blencv  = 1
! vcopt%thetav  = 0.5
! vcopt%ibdtso  = 2
call field_set_key_struct_var_cal_opt(ivarfl(isca(1)), vcopt)
#endif

! Mesh velocity field

#if 0
call field_get_key_struct_var_cal_opt(ivarfl(isca(2)), vcopt)
! vcopt%iconv   = 1
! vcopt%ischcv  = 1
! vcopt%blencv  = 1
! vcopt%thetav  = 0.5
! vcopt%ibdtso  = 2
call field_set_key_struct_var_cal_opt(ivarfl(isca(2)), vcopt)
#endif

! iflxmw = 0

! Mesh viscosity
! iortvm = 0 ! Isotropic
iortvm = 1 ! Orthotropic

return
end subroutine usipsu


!===============================================================================

!> \brief User subroutine for the input of additional user parameters for
!>        input/output.

!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]     nmodpp       number of active specific physics models
!______________________________________________________________________________!

subroutine usipes &
 ( nmodpp )

!===============================================================================
! Module files
!===============================================================================

use paramx
use cstnum
use dimens
use numvar
use optcal
use cstphy
use entsor
use field
use parall
use period
use ihmpre
use post
use ppppar
use ppthch
use ppincl
use cs_c_bindings

!===============================================================================

implicit none

! Arguments

integer nmodpp

! 06/08/18 - Local variables for fields post-processing

integer ii
integer f_id, idim1, ifllog

type(var_cal_opt) :: vcopt

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! Post-processing
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! Post-processing parameters

! Writting time step of the outputs
! -> every 10 times steps
! emphis = "../RESU/postprocessing"
! nthist = 1
! frhist = 1
! tplfmt = 1

! post_activate_by_time_step ()

! Mesh deformation post-processing over time
! ihistr  = 1
! call cs_post_set_deformable ()
 
! Post-processing of varables and properties -> Pressure field
! 1 to activate postprocessing output, 0 to deactivate

! f_id = ivarfl(ipr)
! call field_set_key_int(f_id, keyvis, 1)

! Yplus
f_id = iprpfl(iyplbr)
call field_set_key_int(f_id, keyvis, 0)
call field_set_key_int(f_id, keylog, 0)

! Courant-Friedrich Number (CFL)
f_id = iprpfl(icour)
call field_set_key_int(f_id, keyvis, 0)
call field_set_key_int(f_id, keylog, 0)

! Fourier number
f_id = iprpfl(ifour)
call field_set_key_int(f_id, keyvis, 0)
call field_set_key_int(f_id, keylog, 0)

! Laminar viscosity
f_id = iprpfl(iviscl)
call field_set_key_int(f_id, keyvis, 0)
call field_set_key_int(f_id, keylog, 0)

!#if 0
! Mesh viscosity
f_id = iprpfl(ivisma)
call field_set_key_int(f_id, keyvis, 0)
call field_set_key_int(f_id, keylog, 0)
!#endif

! Total pressure
f_id = iprpfl(iprtot)
call field_set_key_int(f_id, keyvis, 0)
call field_set_key_int(f_id, keylog, 0) 

call cs_user_postprocess_activate(ntmabs , ntcabs, ttcabs)

return
end subroutine usipes


!===============================================================================

